@@ -1,6 +1,6 @@
 package Class::Meta::AccessorBuilder;
 
-# $Id: AccessorBuilder.pm,v 1.10 2004/01/17 19:50:24 david Exp $
+# $Id: AccessorBuilder.pm,v 1.13 2004/01/20 21:34:48 david Exp $
 
 =head1 NAME
 
@@ -90,11 +90,44 @@ way designed by Class::Data::Inheritable, although this might be changed in a
 future release. For now, I expect that the current simple approach will cover
 the vast majority of circumstances.
 
+=head1 Private and Protected Attributes
+
+Any attributes that have their C<view> attribute set to Class::Meta::Private
+or Class::Meta::Protected get additional validation installed to ensure that
+they're truly private and protected. This includes when they are set via
+parameters to constructors generated by Class::Meta. The validation is
+performed by checking the caller of the accessors, and throwing an exception
+when the caller isn't the class that owns the attribute (for private
+attributes) or when it doesn't inherit from the class that owns the attribute
+(for protected attributes).
+
+As an implementation note, this validation is performed for parameters passed
+to constructors created by Class::Meta by ignoring looking for the first
+caller that isn't Class::Meta::Constructor:
+
+  my $caller = caller;
+  # Circumvent generated constructors.
+  for (my $i = 1; $caller eq 'Class::Meta::Constructor'; $i++) {
+      $caller = caller($i);
+  }
+
+This works because Class::Meta::Constructor installs the closures that become
+constructors, and thus, when those closures call accessors to set new values
+for attributes, the caller is Class::Meta::Constructor. By going up the stack
+until we find another package, we correctly check to see what context is
+setting attribute values via a constructor, rather than the constructor method
+itself being the context.
+
+This is a bit of a hack, but since Perl uses call stacks for checking security
+in this way, it's the best I could come up with. Other suggestions welcome. Or
+see L<Class::Meta::Type|Class::Meta::Type/"Custom Accessor Building"> to
+create your own accessor generation code
+
 =cut
 
 use strict;
 use Class::Meta;
-our $VERSION = "0.12";
+our $VERSION = "0.13";
 
 sub build_attr_get {
     UNIVERSAL::can($_[0]->package, $_[0]->name);
@@ -104,7 +137,7 @@ sub build_attr_get {
 
 my $croak = sub {
     require Carp;
-    our @CARP_NOT = qw(Class::Meta::Attribute);
+    our @CARP_NOT = qw(Class::Meta::Attribute Class::Meta::Constructor);
     Carp::croak(@_);
 };
 
@@ -114,26 +147,25 @@ my $req_chk = sub {
 
 sub build {
     my ($pkg, $attr, $create, @checks) = @_;
-    unshift @checks, $req_chk if $attr->required;
     my $name = $attr->name;
 
-    # XXX Do I need to add code to check the caller and throw an exception for
-    # private and protected methods?
+    # Add the required check, if needed.
+    unshift @checks, $req_chk if $attr->required;
 
-    no strict 'refs';
-    if ($attr->context == Class::Meta::OBJECT) {
+    my $sub;
+    if ($attr->context == Class::Meta::CLASS) {
         # Create class attribute accessors by creating a closure tha
         # references this variable.
         my $data = $attr->default;
 
         if ($create == Class::Meta::GET) {
             # Create GET accessor.
-            *{"${pkg}::$name"} = sub { $data };
+            $sub = sub { $data };
 
         } elsif ($create == Class::Meta::SET) {
             # Create SET accessor.
             if (@checks) {
-                *{"${pkg}::$name"} = sub {
+                $sub = sub {
                     # Check the value passed in.
                     $_->($_[1]) for @checks;
                     # Assign the value.
@@ -141,7 +173,7 @@ sub build {
                     return;
                 };
             } else {
-                *{"${pkg}::$name"} = sub {
+                $sub = sub {
                     # Assign the value.
                     $data = $_[1];
                     return;
@@ -151,7 +183,7 @@ sub build {
         } elsif ($create == Class::Meta::GETSET) {
             # Create GETSET accessor(s).
             if (@checks) {
-                *{"${pkg}::$name"} = sub {
+                $sub = sub {
                     my $self = shift;
                     return $data unless @_;
                     # Check the value passed in.
@@ -160,7 +192,7 @@ sub build {
                     return $data = $_[0];
                 };
             } else {
-                *{"${pkg}::$name"} = sub {
+                $sub = sub {
                     my $self = shift;
                     return $data unless @_;
                     # Assign the value.
@@ -170,54 +202,84 @@ sub build {
         } else {
             # Well, nothing I guess.
         }
-        return
-    }
-
-    # Create object attribute accessors.
-    if ($create == Class::Meta::GET) {
-        # Create GET accessor.
-        *{"${pkg}::$name"} = sub { $_[0]->{$name} };
-
-    } elsif ($create == Class::Meta::SET) {
-        # Create SET accessor.
-        if (@checks) {
-            *{"${pkg}::$name"} = sub {
-                # Check the value passed in.
-                $_->($_[1]) for @checks;
-                # Assign the value.
-                $_[0]->{$name} = $_[1];
-                return;
-            };
-        } else {
-            *{"${pkg}::$name"} = sub {
-                # Assign the value.
-                $_[0]->{$name} = $_[1];
-                return;
-            };
-        }
-
-    } elsif ($create == Class::Meta::GETSET) {
-        # Create GETSET accessor(s).
-        if (@checks) {
-            *{"${pkg}::$name"} = sub {
-                my $self = shift;
-                return $self->{$name} unless @_;
-                # Check the value passed in.
-                $_->($_[0]) for @checks;
-                # Assign the value.
-                return $self->{$name} = $_[0];
-            };
-        } else {
-            *{"${pkg}::$name"} = sub {
-                my $self = shift;
-                return $self->{$name} unless @_;
-                # Assign the value.
-                return $self->{$name} = shift;
-            };
-        }
     } else {
-        # Well, nothing I guess.
+        # Create object attribute accessors.
+        if ($create == Class::Meta::GET) {
+            # Create GET accessor.
+            $sub = sub { $_[0]->{$name} };
+
+        } elsif ($create == Class::Meta::SET) {
+            # Create SET accessor.
+            if (@checks) {
+                $sub = sub {
+                    # Check the value passed in.
+                    $_->($_[1]) for @checks;
+                    # Assign the value.
+                    $_[0]->{$name} = $_[1];
+                    return;
+                };
+            } else {
+                $sub = sub {
+                    # Assign the value.
+                    $_[0]->{$name} = $_[1];
+                    return;
+                };
+            }
+
+        } elsif ($create == Class::Meta::GETSET) {
+            # Create GETSET accessor(s).
+            if (@checks) {
+                $sub = sub {
+                    my $self = shift;
+                    return $self->{$name} unless @_;
+                    # Check the value passed in.
+                    $_->($_[0]) for @checks;
+                    # Assign the value.
+                    return $self->{$name} = $_[0];
+                };
+            } else {
+                $sub = sub {
+                    my $self = shift;
+                    return $self->{$name} unless @_;
+                    # Assign the value.
+                    return $self->{$name} = shift;
+                };
+            }
+        } else {
+            # Well, nothing I guess.
+        }
     }
+
+    # Add public an private checks, if required.
+    if ($attr->view == Class::Meta::PROTECTED) {
+        my $real_sub = $sub;
+         $sub = sub {
+             my $caller = caller;
+             # Circumvent generated constructors.
+             for (my $i = 1; $caller eq 'Class::Meta::Constructor'; $i++) {
+                 $caller = caller($i);
+             }
+             $croak->("$name is a protected attribute of $pkg!")
+               unless UNIVERSAL::isa($caller, $pkg);
+             goto &$real_sub;
+        };
+    } elsif ($attr->view == Class::Meta::PRIVATE) {
+        my $real_sub = $sub;
+        $sub = sub {
+             my $caller = caller;
+             # Circumvent generated constructors.
+             for (my $i = 1; $caller eq 'Class::Meta::Constructor'; $i++) {
+                 $caller = caller($i);
+             }
+             $croak->("$name is a private attribute of $pkg!")
+               unless $caller eq $pkg;
+             goto &$real_sub;
+         };
+    }
+
+    # Install the accessor.
+    no strict 'refs';
+    *{"${pkg}::$name"} = $sub;
 }
 
 1;
@@ -225,7 +287,7 @@ __END__
 
 =head1 DISTRIBUTION INFORMATION
 
-This file was packaged with the Class-Meta-0.11 distribution.
+This file was packaged with the Class-Meta-0.13 distribution.
 
 =head1 BUGS
 
